package com.snail.exam.s2dao;

import java.sql.Timestamp;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TradeModel {

    private static final Log log = LogFactory.getLog(TradeModel.class);
    
    private CustomerDao pCustomerDao;
    private ItemDao pItemDao;
    private OrderDao pOrderDao;

    public TradeModel(CustomerDao customerDao,ItemDao itemDao,OrderDao orderDao) {
        super();
        
        pCustomerDao = customerDao;
        pItemDao = itemDao;
        pOrderDao = orderDao;
    }
    
    public boolean checkCreditFacility(long userId){
        
        int currentCreditFacility = pOrderDao.sumCreditSale(userId);
        log.debug("現在の顧客番号"+userId+"の売掛金は"+currentCreditFacility+"です。");
        
        CustomerBean customer = pCustomerDao.findCustomer(userId);
        int maxCreditFacility = customer.getCreditFacility();
        log.debug("現在の顧客番号"+userId+"の与信枠は"+maxCreditFacility+"です。");
        
        return currentCreditFacility < maxCreditFacility;
    }
    
    public boolean checkStock(long itemId,int amount ){
        
        ItemBean item = pItemDao.findItemByPK(itemId);
        
        log.debug(item.getName() + "の在庫:" + item.getStock());
        
        return amount < item.getStock();
    }
    
    public long setAside(long userId,long itemId,int amount){
        
        CustomerBean customer = pCustomerDao.findCustomer(userId);
        ItemBean item = pItemDao.findItemByPK(itemId);
        
        // 在庫を減らす
        item.setStock(item.getStock()-amount);
        pItemDao.update(item);
        
        // 受注をたてる
        OrderBean order = new OrderBean();
        order.setAmount(amount);
        order.setCustomerId(customer.getId());
        order.setItemId(item.getId());
        Timestamp now = new Timestamp(System.currentTimeMillis());
        order.setOnCreate(now);
        order.setOnUpdate(now);
        order.setStatus('0');
        order.setTradePrice((int)(item.getPrice() * (1.0 - customer.getDiscountRate())));
        pOrderDao.insert(order);
        
        return order.getId();
    }

}
